
#include "auth.h"
#include "debug.h"
#include "xmalloc.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <sys/stat.h>

static int auth_filesystem_assert( struct link *link, time_t stoptime )
{
	int success=0;
	FILE *file;
	char line[AUTH_LINE_MAX];

	debug("auth_filesystem: waiting for challenge");
	if(link_readline(link,line,sizeof(line),stoptime)) {
		debug("auth_filesystem: challenge is %s",line);
		file = fopen(line,"w");
		if(file) {
			fsync(fileno(file));
			fclose(file);
			debug("auth_filesystem: issued response");
			if(auth_barrier(link,"yes\n",stoptime)) {
				debug("auth_filesystem: response accepted");
				success = 1;
			} else {
				debug("auth_filesystem: response rejected");
			}
		} else {
			debug("auth_filesystem: could not meet challenge: %s",strerror(errno));
			auth_barrier(link,"no\n",stoptime);
		}
	} else {
		debug("auth_filesystem: couldn't read challenge");
	}

	return success;
}

static const char *challenge_dir=0;

static void make_challenge_path( char *path )
{
	const char *dir;
	int result;

	if(challenge_dir) {
		dir = challenge_dir;
	} else {
		dir = "/tmp";
	}

	while(1) {
		sprintf(path,"%s/challenge.%d.%d",dir,(int)getpid(),rand());
		result = unlink(path);
		if(result==0) {
			break;
		} else {
			if(errno==ENOENT) {
				break;
			} else {
				debug("auth_filesystem: %s is in use, still trying...");
				continue;
			}
		}
	}

	debug("auth_filesystem: challenge path is %s",path);
}

static int auth_filesystem_accept( struct link *link, char **subject, time_t stoptime )
{
	char path[AUTH_LINE_MAX];
	char line[AUTH_LINE_MAX];
	int success=0;
	struct stat buf;
	struct passwd *p;

	debug("auth_filesystem: generating challenge");
	make_challenge_path(path);
	link_write(link,path,strlen(path),stoptime);
	link_write(link,"\n",1,stoptime);
	
	debug("auth_filesystem: waiting for response");
	if(link_readline(link,line,sizeof(line),stoptime)) {
		if(!strcmp(line,"yes")) {
			if(stat(path,&buf)==0) {
				debug("auth_filesystem: client is uid %d",buf.st_uid);
				p = getpwuid(buf.st_uid);
				if(p) {
					debug("auth_filesystem: client is subject %s",p->pw_name);
					unlink(path);
					link_write(link,"yes\n",4,stoptime);
					*subject = xstrdup(p->pw_name);
					success = 1;
				} else {
					debug("auth_filesystem: there is no user corresponding to uid %d",buf.st_uid);
					link_write(link,"no\n",3,stoptime);
				}
			} else {
				debug("auth_filesystem: client failed the challenge: %s",strerror(errno));
			}
		} else {
			debug("auth_filesystem: client declined the challenge");
		}
	}

	return success;
}

int auth_filesystem_register( const char *challenge_dir )
{
	debug("auth_filesystem: registered");
	return auth_register( "filesystem", auth_filesystem_assert, auth_filesystem_accept );
}

