/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "debug.h"
#include "config.h"
#include "config_table.h"
#include "int_sizes.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

static struct config_table *table=0;

int config_init( const char *filename )
{
	char *s;
	INT64_T size;

	if(!filename) return 0;

	table = config_table_create();
	if(!table) return 0;

	if(!config_table_load(table,filename)) {
		config_table_delete(table);
		return 0;
	}

	if(config_get("debug_to_console",&s)) {
		debug_config_console(1);
		free(s);
	}

	if(config_get("debug_to_file",&s)) {
		debug_config_file(s);
		free(s);
	}

	if(config_get_int("debug_to_file_size",&size)) {
		debug_config_file_size(size);
	}

	if(config_get("debug_to_host",&s)) {
		debug_config_host(s);
		free(s);
	}

	return 1;
}

int config_get( const char *name, char **result )
{
	return config_table_get(table,name,result);
}

int config_get_int( const char *name, INT64_T *result )
{
	return config_table_get_int(table,name,result);
}

char * config_require( const char *name )
{
	char *r;

	if(!config_table_get(table,name,&r))
		fatal("config: %s is not defined!",name);

	return r;
}

INT64_T config_require_int( const char *name )
{
	INT64_T r;

	if(!config_table_get_int(table,name,&r))
		fatal("config: %s is not defined!",name);

	return r;
}



