/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "memory_manager.h"

#include <pthread.h>
#include <stdlib.h>

struct memory_manager {
	msize_t max;
	msize_t inuse;
	pthread_mutex_t mutex;
	pthread_cond_t cond;
};


struct memory_manager * memory_manager_create( msize_t max )
{
	struct memory_manager *m;

	m = malloc(sizeof(*m));
	if(!m) return 0;

	m->max = max;
	m->inuse = 0;
	pthread_cond_init(&m->cond,0);
	pthread_mutex_init(&m->mutex,0);

	return m;
}

void memory_manager_delete( struct memory_manager *mm )
{
	if(mm) free(mm);
}

void memory_manager_alloc( struct memory_manager *mm, msize_t size )
{
	pthread_mutex_lock(&mm->mutex);
	while( (mm->inuse+size) > mm->max ) {
		pthread_cond_wait(&mm->cond,&mm->mutex);
	}
	mm->inuse += size;
	pthread_mutex_unlock(&mm->mutex);
}

void memory_manager_free( struct memory_manager *mm, msize_t size )
{
	pthread_mutex_lock(&mm->mutex);
	mm->inuse -= size;
	pthread_cond_broadcast(&mm->cond);
	pthread_mutex_unlock(&mm->mutex);
}

void memory_manager_set( struct memory_manager *mm, msize_t max )
{
	pthread_mutex_lock(&mm->mutex);
	mm->max = max;
	pthread_cond_broadcast(&mm->cond);
	pthread_mutex_unlock(&mm->mutex);
}

void memory_manager_get( struct memory_manager *mm, msize_t *max, msize_t *inuse )
{
	pthread_mutex_lock(&mm->mutex);
	*max = mm->max;
	*inuse = mm->inuse;
	pthread_mutex_unlock(&mm->mutex);
}

void memory_manager_add( struct memory_manager *mm, msize_t extra )
{
	pthread_mutex_lock(&mm->mutex);
	mm->inuse += extra;
	pthread_cond_broadcast(&mm->cond);
	pthread_mutex_unlock(&mm->mutex);
}
