/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "pvalue.h"
#include "safe_file.h"

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>

int vfscanf( FILE *stream, const char *format, va_list ap );

int pvalue_get( const char *filename, const char *fmt, ... )
{
	va_list args;
	va_start(args,fmt);

	return pvalue_vget(filename,fmt,args);

	va_end(args);
}

int pvalue_set( const char *filename, const char *fmt, ... )
{
	va_list args;
	va_start(args,fmt);

	return pvalue_vset(filename,fmt,args);

	va_end(args);
}

int pvalue_vget( const char *filename, const char *fmt, va_list args )
{
	FILE *file;
	int result;

	file = fopen(filename,"r");
	if(!file) return 0;

	result = vfscanf(file,fmt,args);
	fclose(file);

	if(result==1) {
		return 1;
	} else {
		return 0;
	}
}

int pvalue_vset( const char *filename, const char *fmt, va_list args )
{
	struct safe_file *s;
	int result;

	s = safe_file_begin(filename);
	if(!s) return 0;

	result = vfprintf(safe_file_stream(s),fmt,args);
	if(result<=0) {
		safe_file_abort(s);
		return 0;
	}

	return safe_file_commit(s);
}

