/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


/* This module has not been ported everywhere yet. */

#ifdef linux

#include "sound.h"
#include "int_sizes.h"

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <stdio.h>

struct encoding_type {
	int encoding;
	int bits_per_sample;
	int format;
};

struct encoding_type types[] = {
	{SOUND_ENCODING_ULAW,8,AFMT_MU_LAW},
	{SOUND_ENCODING_ALAW,8,AFMT_A_LAW},
	{SOUND_ENCODING_ADPCM,4,AFMT_IMA_ADPCM},
	{SOUND_ENCODING_MPEG,0,AFMT_MPEG},
	{SOUND_ENCODING_LINEAR,8,AFMT_U8},
	{SOUND_ENCODING_LINEAR,16,AFMT_S16_LE},
	{-1,-1,-1},
};

int sound_open( struct sound_config *config )
{
	int fd;
	int result;
	struct encoding_type *e;
	int format=-1;
	UINT32_T fraginfo;
	UINT16_T numfrags, fragsize, fragpow;

	fd = open("/dev/dsp",O_RDWR);
	if(fd<0) return -1;

	/* We want a short fragment size which allows reading in chunks of about .1 secs.  There is also no need to buffer up more than one second of sound, as we don't ever want to get too far behind.  So, in the upper 16 bits, encode ten fragments, and in the lower, encode .1 secs worth of data. */

	numfrags = 10;
	fragsize = (config->samples_per_second*config->bits_per_sample/8)/10;
	fragpow = 1;

	while(fragsize>2) {
		fragpow++;
		fragsize/=2;
	}

	fraginfo = (numfrags<<16) | fragpow;

	result = ioctl(fd,SNDCTL_DSP_SETFRAGMENT,&fraginfo);
	if(result<0) {
		printf("Warning: Couldn't set audio fragment size.  You may experience delayed sound.\n");
	}

	/* Now, search for a format setting which matches the encoding and bits given in the config */

	for(e=types;e->encoding!=-1;e++) {
		if( (config->encoding==e->encoding) && (config->bits_per_sample==e->bits_per_sample) ) {
			format = e->format;
			break;
		}
	}

	if(format==-1) return 0;

	result = ioctl(fd,SNDCTL_DSP_SETFMT,&format);
	if( result<0 || format!=e->format ) {
		close(fd);
		return -1;
	}

	result = ioctl(fd,SNDCTL_DSP_SPEED,&config->samples_per_second);
	if(result<0) {
		close(fd);
		return -1;
	}

	return fd;
}

int sound_record( int fd, void *data, int length )
{
	return read(fd,data,length);
}

int sound_play( int fd, const void *data, int length )
{
	return write(fd,data,length);
}

int sound_flush( int fd )
{
	int result;
	result = ioctl(fd,SNDCTL_DSP_POST);
	if(result<0) return 0;
	return 1;
}

int sound_close( int fd )
{
	return close(fd);
}

#endif
